Package: CompRiskRel
Type: Package
Title: Reliability and Competing Risks Analysis under Hybrid Censoring
Version: 0.1.0
Authors@R: c(
    person(given = "Shikhar",
           family = "Tyagi",
           role = c("aut", "cre"),
           email = "shikhar1093tyagi@gmail.com",
           comment = c(ORCID = "0000-0003-1606-0844")),
    person(given = "Arvind",
           family = "Pandey",
           role = "aut",
           email = "arvindmzu@gmail.com"),
    person(given = "Bhupendra",
           family = "Singh",
           role = "aut",
           email = "bhupendra.rana@gmail.com"),
    person(given = "Vrijesh",
           family = "Tripathi",
           role = "aut",
           email = "vrijesh.tripathi@uwi.edu"))
Description: Generalized computational algorithms for competing risks analysis,
    stress-strength reliability modeling, optimal designs, and reliability
    acceptance sampling plans under various hybrid censoring schemes. Includes
    data generation routines, Maximum Likelihood Estimation (MLE) with seven
    optimization algorithms ('Newton-Raphson', 'BFGS', 'BFGSR', 'BHHH', 'SANN',
    'CG', and 'Nelder-Mead'), Bayesian inference via Gibbs sampling and
    Metropolis-Hastings MCMC, Importance Sampling, and 'Lindley' asymptotic
    approximation. Visualization functions generate histograms, dot plots,
    and autocorrelation plots for model validation. Methodology and design
    principles are based on 'Balakrishnan', 'Cramer', and 'Kundu' (2023,
    "Hybrid Censoring Know-How: Designs and Implementations", Academic Press,
    ISBN:978-0-12-398387-9).
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, graphics
Suggests: testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2026-07-25 17:29:46 UTC; shikhar tyagi
Author: Shikhar Tyagi [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1606-0844>),
  Arvind Pandey [aut],
  Bhupendra Singh [aut],
  Vrijesh Tripathi [aut]
Maintainer: Shikhar Tyagi <shikhar1093tyagi@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-05 06:30:11 UTC
Built: R 4.6.0; ; 2026-08-05 08:51:33 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,comp_risk_rel_data)
S3method(print,bayes_fit)
S3method(print,mle_fit)
S3method(print,sampling_plan)
S3method(summary,bayes_fit)
S3method(summary,mle_fit)
export(bayes_gibbs_censored)
export(bayes_mh_censored)
export(bayesian_sampling_plan_exp)
export(gen_competing_risks)
export(gen_gen_prog_hybrid)
export(gen_stress_strength)
export(gen_type1_hybrid)
export(gen_type2_hybrid)
export(importance_sampling_censored)
export(lindley_approx_censored)
export(mle_competing_risks)
export(mle_gen_prog_hybrid)
export(mle_type1_hybrid)
export(mle_type2_hybrid)
export(optimal_design_hybrid)
export(plot_diagnostics)
export(sampling_plan_exponential)
export(sampling_plan_weibull)
export(stress_strength_rel)

#' Generate Data under Type-I Hybrid Censoring Scheme
#'
#' @param pdf Probability density function.
#' @param cdf Cumulative distribution function.
#' @param lower Lower bound of the support of the distribution.
#' @param upper Upper bound of the support of the distribution.
#' @param n Total sample size.
#' @param r Minimum required number of failures.
#' @param T_star Fixed termination time.
#' @param seed Optional integer random seed for reproducibility.
#'
#' @return A list containing:
#' \item{observed_times}{Vector of observed failure/censoring times.}
#' \item{censor_status}{Binary indicator (1 for failure, 0 for censored at T_star).}
#' \item{n_failures}{Number of observed failures.}
#' \item{termination_time}{Effective termination time of the test.}
#' \item{scheme}{Character string indicating censoring scheme.}
#' @export
#'
#' @examples
#' gen_type1_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, r = 10,
#'   T_star = 1.5, seed = 123
#' )
gen_type1_hybrid <- function(pdf, cdf, lower = 0, upper = Inf, n, r, T_star, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  u <- stats::runif(n)
  inv_cdf <- function(p) {
    sapply(p, function(pv) {
      if (pv <= 0) return(lower)
      if (pv >= 1) return(upper)
      res <- tryCatch(
        stats::uniroot(function(x) cdf(x) - pv, lower = lower, upper = ifelse(is.infinite(upper), 1e5, upper))$root,
        error = function(e) NA
      )
      if (is.na(res)) lower else res
    })
  }
  x <- sort(inv_cdf(u))
  # Type-I Hybrid: test stops at T_stop = min(X_(r), T_star)
  x_r <- if (r <= n) x[r] else Inf
  T_stop <- min(x_r, T_star)
  
  obs_times <- pmin(x, T_stop)
  censor_status <- ifelse(x <= T_stop, 1, 0)
  obs_times <- obs_times[censor_status == 1 | obs_times <= T_stop]
  
  res <- list(
    observed_times = obs_times,
    censor_status = censor_status,
    n_failures = sum(censor_status),
    termination_time = T_stop,
    scheme = "Type-I Hybrid Censoring"
  )
  class(res) <- "comp_risk_rel_data"
  return(res)
}

#' Generate Data under Type-II Hybrid Censoring Scheme
#'
#' @param pdf Probability density function.
#' @param cdf Cumulative distribution function.
#' @param lower Lower bound of the support of the distribution.
#' @param upper Upper bound of the support of the distribution.
#' @param n Total sample size.
#' @param r Minimum required number of failures.
#' @param T_star Target observation time limit.
#' @param seed Optional integer random seed for reproducibility.
#'
#' @return A list containing observed failure times, censor statuses, and termination details.
#' @export
#'
#' @examples
#' gen_type2_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, r = 10,
#'   T_star = 1.0, seed = 123
#' )
gen_type2_hybrid <- function(pdf, cdf, lower = 0, upper = Inf, n, r, T_star, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  u <- stats::runif(n)
  inv_cdf <- function(p) {
    sapply(p, function(pv) {
      if (pv <= 0) return(lower)
      if (pv >= 1) return(upper)
      res <- tryCatch(
        stats::uniroot(function(x) cdf(x) - pv, lower = lower, upper = ifelse(is.infinite(upper), 1e5, upper))$root,
        error = function(e) NA
      )
      if (is.na(res)) lower else res
    })
  }
  x <- sort(inv_cdf(u))
  # Type-II Hybrid: test stops at T_stop = max(X_(r), T_star)
  x_r <- if (r <= n) x[r] else x[n]
  T_stop <- max(x_r, T_star)
  
  obs_times <- x[x <= T_stop]
  censor_status <- rep(1, length(obs_times))
  
  res <- list(
    observed_times = obs_times,
    censor_status = censor_status,
    n_failures = length(obs_times),
    termination_time = T_stop,
    scheme = "Type-II Hybrid Censoring"
  )
  class(res) <- "comp_risk_rel_data"
  return(res)
}

#' Generate Data under Generalized Progressive Hybrid Censoring Scheme
#'
#' @param pdf Probability density function.
#' @param cdf Cumulative distribution function.
#' @param lower Lower bound of support.
#' @param upper Upper bound of support.
#' @param n Total sample size.
#' @param m Target number of failures.
#' @param k Threshold failure count.
#' @param T_star Termination time limit.
#' @param R_plan Progressive censoring plan vector of length m.
#' @param seed Optional integer random seed for reproducibility.
#'
#' @return A list containing observed failure times, progressive removal plan, and censoring details.
#' @export
#'
#' @examples
#' gen_gen_prog_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, m = 10,
#'   k = 5, T_star = 1.2, R_plan = rep(1, 10),
#'   seed = 123
#' )
gen_gen_prog_hybrid <- function(pdf, cdf, lower = 0, upper = Inf, n, m, k, T_star, R_plan, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  if (length(R_plan) != m) R_plan <- rep(floor((n - m) / m), m)
  
  u <- stats::runif(n)
  inv_cdf <- function(p) {
    sapply(p, function(pv) {
      if (pv <= 0) return(lower)
      if (pv >= 1) return(upper)
      res <- tryCatch(
        stats::uniroot(function(x) cdf(x) - pv, lower = lower, upper = ifelse(is.infinite(upper), 1e5, upper))$root,
        error = function(e) NA
      )
      if (is.na(res)) lower else res
    })
  }
  x <- sort(inv_cdf(u))
  
  # Progressive Type-II sampling simulation
  obs_times <- numeric(m)
  rem_pool <- x
  for (i in 1:m) {
    if (length(rem_pool) == 0) break
    obs_times[i] <- rem_pool[1]
    rem_pool <- rem_pool[-1]
    r_i <- R_plan[i]
    if (r_i > 0 && length(rem_pool) >= r_i) {
      drop_idx <- sample(seq_along(rem_pool), r_i)
      rem_pool <- rem_pool[-drop_idx]
    }
  }
  
  # Apply time cutoff T_star according to generalized progressive hybrid scheme
  D <- sum(obs_times <= T_star)
  if (D >= m) {
    final_obs <- obs_times[1:m]
    censor_status <- rep(1, m)
    T_stop <- obs_times[m]
  } else if (D >= k && D < m) {
    final_obs <- obs_times[1:D]
    censor_status <- rep(1, D)
    T_stop <- T_star
  } else {
    eff_m <- min(k, length(obs_times))
    final_obs <- obs_times[1:eff_m]
    censor_status <- rep(1, eff_m)
    T_stop <- obs_times[eff_m]
  }
  
  res <- list(
    observed_times = final_obs,
    censor_status = censor_status,
    n_failures = length(final_obs),
    termination_time = T_stop,
    scheme = "Generalized Progressive Hybrid Censoring"
  )
  class(res) <- "comp_risk_rel_data"
  return(res)
}

#' Generate Data for Competing Risks Analysis
#'
#' @param pdf1 Density function for Cause 1.
#' @param cdf1 Cumulative distribution function for Cause 1.
#' @param pdf2 Density function for Cause 2.
#' @param cdf2 Cumulative distribution function for Cause 2.
#' @param lower Lower bound of support.
#' @param upper Upper bound of support.
#' @param n Total sample size.
#' @param censoring_type Type of censoring ("type1_hybrid", "type2_hybrid", "prog_hybrid").
#' @param r Number of failures required.
#' @param T_star Time limit.
#' @param R_plan Progressive plan (if applicable).
#' @param seed Optional random seed.
#'
#' @return A list containing observed failure times, causes of failure (1 or 2), and censoring flags.
#' @export
#'
#' @examples
#' gen_competing_risks(
#'   pdf1 = function(x) dexp(x, rate = 1),
#'   cdf1 = function(x) pexp(x, rate = 1),
#'   pdf2 = function(x) dexp(x, rate = 1.5),
#'   cdf2 = function(x) pexp(x, rate = 1.5),
#'   lower = 0, upper = 10, n = 25,
#'   censoring_type = "type1_hybrid",
#'   r = 15, T_star = 1.0, seed = 123
#' )
gen_competing_risks <- function(pdf1, cdf1, pdf2, cdf2, lower = 0, upper = Inf, n,
                                censoring_type = c("type1_hybrid", "type2_hybrid", "prog_hybrid"),
                                r = NULL, T_star = NULL, R_plan = NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  censoring_type <- match.arg(censoring_type)
  
  # Generate latent failure times X1 and X2 under independent competing risks
  inv_cdf1 <- function(p) sapply(p, function(pv) stats::uniroot(function(x) cdf1(x) - pv, lower = lower, upper = ifelse(is.infinite(upper), 1e5, upper))$root)
  inv_cdf2 <- function(p) sapply(p, function(pv) stats::uniroot(function(x) cdf2(x) - pv, lower = lower, upper = ifelse(is.infinite(upper), 1e5, upper))$root)
  
  x1 <- inv_cdf1(stats::runif(n))
  x2 <- inv_cdf2(stats::runif(n))
  
  x_min <- pmin(x1, x2)
  cause <- ifelse(x1 <= x2, 1, 2)
  
  ord <- order(x_min)
  x_min <- x_min[ord]
  cause <- cause[ord]
  
  if (censoring_type == "type1_hybrid") {
    r_val <- if (is.null(r)) floor(n / 2) else r
    t_val <- if (is.null(T_star)) stats::median(x_min) else T_star
    T_stop <- min(x_min[r_val], t_val)
    idx <- which(x_min <= T_stop)
    obs_t <- x_min[idx]
    obs_cause <- cause[idx]
  } else if (censoring_type == "type2_hybrid") {
    r_val <- if (is.null(r)) floor(n / 2) else r
    t_val <- if (is.null(T_star)) stats::median(x_min) else T_star
    T_stop <- max(x_min[r_val], t_val)
    idx <- which(x_min <= T_stop)
    obs_t <- x_min[idx]
    obs_cause <- cause[idx]
  } else {
    m_val <- if (is.null(r)) floor(n / 2) else r
    obs_t <- x_min[1:m_val]
    obs_cause <- cause[1:m_val]
    T_stop <- x_min[m_val]
  }
  
  res <- list(
    observed_times = obs_t,
    causes = obs_cause,
    n_failures = length(obs_t),
    termination_time = T_stop,
    scheme = paste("Competing Risks -", censoring_type)
  )
  class(res) <- "comp_risk_rel_data"
  return(res)
}

#' Generate Data for Stress-Strength Reliability Models
#'
#' @param pdf_X Density function for stress X.
#' @param cdf_X Cumulative distribution function for stress X.
#' @param pdf_Y Density function for strength Y.
#' @param cdf_Y Cumulative distribution function for strength Y.
#' @param lower_X Lower bound for X.
#' @param upper_X Upper bound for X.
#' @param lower_Y Lower bound for Y.
#' @param upper_Y Upper bound for Y.
#' @param n_X Sample size for stress X.
#' @param n_Y Sample size for strength Y.
#' @param censoring_type Censoring scheme.
#' @param r_X Minimum required failures for X.
#' @param T_X Time cutoff for X.
#' @param r_Y Minimum required failures for Y.
#' @param T_Y Time cutoff for Y.
#' @param seed Optional random seed.
#'
#' @return A list containing generated stress dataset X and strength dataset Y.
#' @export
#'
#' @examples
#' gen_stress_strength(
#'   pdf_X = function(x) dexp(x, rate = 1.2),
#'   cdf_X = function(x) pexp(x, rate = 1.2),
#'   pdf_Y = function(y) dexp(y, rate = 0.8),
#'   cdf_Y = function(y) pexp(y, rate = 0.8),
#'   n_X = 20, n_Y = 20,
#'   censoring_type = "type1_hybrid",
#'   r_X = 12, T_X = 1.2, r_Y = 12, T_Y = 1.5, seed = 123
#' )
gen_stress_strength <- function(pdf_X, cdf_X, pdf_Y, cdf_Y,
                                lower_X = 0, upper_X = Inf, lower_Y = 0, upper_Y = Inf,
                                n_X, n_Y, censoring_type = c("type1_hybrid", "type2_hybrid", "prog_hybrid"),
                                r_X = NULL, T_X = NULL, r_Y = NULL, T_Y = NULL, seed = NULL) {
  if (!is.null(seed)) set.seed(seed)
  censoring_type <- match.arg(censoring_type)
  
  rx <- if (is.null(r_X)) floor(n_X * 0.7) else r_X
  tx <- if (is.null(T_X)) 1.5 else T_X
  ry <- if (is.null(r_Y)) floor(n_Y * 0.7) else r_Y
  ty <- if (is.null(T_Y)) 1.5 else T_Y
  
  data_X <- if (censoring_type == "type1_hybrid") {
    gen_type1_hybrid(pdf_X, cdf_X, lower_X, upper_X, n_X, rx, tx)
  } else {
    gen_type2_hybrid(pdf_X, cdf_X, lower_X, upper_X, n_X, rx, tx)
  }
  
  data_Y <- if (censoring_type == "type1_hybrid") {
    gen_type1_hybrid(pdf_Y, cdf_Y, lower_Y, upper_Y, n_Y, ry, ty)
  } else {
    gen_type2_hybrid(pdf_Y, cdf_Y, lower_Y, upper_Y, n_Y, ry, ty)
  }
  
  res <- list(
    data_X = data_X,
    data_Y = data_Y,
    scheme = paste("Stress-Strength Models -", censoring_type)
  )
  class(res) <- "comp_risk_rel_ss_data"
  return(res)
}

#' Bayesian Inference via Gibbs Sampling for Censored Data Models
#'
#' @param log_posterior Function taking parameter vector and returning log posterior density.
#' @param init_par Initial parameter vector.
#' @param n_sim Total number of MCMC iterations.
#' @param burn_in Number of initial iterations to discard as burn-in.
#'
#' @return S3 object of class \code{bayes_fit} with parameter chains, posterior means, and credible intervals.
#' @export
#'
#' @examples
#' log_post <- function(th) {
#'   if (th[1] <= 0) return(-Inf)
#'   dexp(th[1], rate = 1, log = TRUE) +
#'     sum(dexp(c(0.5, 1.2, 0.8), rate = th[1], log = TRUE))
#' }
#' bayes_gibbs_censored(
#'   log_post, init_par = c(1.0),
#'   n_sim = 1000, burn_in = 200
#' )
bayes_gibbs_censored <- function(log_posterior, init_par, n_sim = 5000, burn_in = 1000) {
  p_dim <- length(init_par)
  chain <- matrix(0, nrow = n_sim, ncol = p_dim)
  chain[1, ] <- init_par
  
  curr_par <- init_par
  for (i in 2:n_sim) {
    for (j in 1:p_dim) {
      cand <- curr_par
      cand[j] <- stats::rnorm(1, mean = curr_par[j], sd = 0.1)
      if (cand[j] > 0) {
        log_alpha <- log_posterior(cand) - log_posterior(curr_par)
        if (!is.na(log_alpha) && log(stats::runif(1)) < log_alpha) {
          curr_par[j] <- cand[j]
        }
      }
    }
    chain[i, ] <- curr_par
  }
  
  post_chain <- chain[(burn_in + 1):n_sim, , drop = FALSE]
  post_means <- colMeans(post_chain)
  ci_lower <- apply(post_chain, 2, function(x) stats::quantile(x, 0.025))
  ci_upper <- apply(post_chain, 2, function(x) stats::quantile(x, 0.975))
  
  res <- list(
    chain = post_chain,
    post_means = post_means,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    method = "Gibbs Sampling"
  )
  class(res) <- "bayes_fit"
  return(res)
}

#' Bayesian Inference via Metropolis-Hastings MCMC Algorithm
#'
#' @param log_posterior Function evaluating log-posterior density.
#' @param init_par Initial parameter values.
#' @param n_sim Total MCMC sample size.
#' @param burn_in Discarded burn-in samples.
#' @param proposal_sd Proposal standard deviation vector.
#'
#' @return S3 object of class \code{bayes_fit}.
#' @export
#'
#' @examples
#' log_post <- function(th) {
#'   if (th[1] <= 0) return(-Inf)
#'   dexp(th[1], rate = 1, log = TRUE) +
#'     sum(dexp(c(0.5, 1.2, 0.8), rate = th[1], log = TRUE))
#' }
#' bayes_mh_censored(
#'   log_post, init_par = c(1.0),
#'   n_sim = 1000, burn_in = 200
#' )
bayes_mh_censored <- function(log_posterior, init_par, n_sim = 5000, burn_in = 1000, proposal_sd = 0.1) {
  p_dim <- length(init_par)
  if (length(proposal_sd) == 1) proposal_sd <- rep(proposal_sd, p_dim)
  
  chain <- matrix(0, nrow = n_sim, ncol = p_dim)
  chain[1, ] <- init_par
  curr_par <- init_par
  curr_lp <- log_posterior(curr_par)
  
  n_accept <- 0
  for (i in 2:n_sim) {
    cand <- curr_par + stats::rnorm(p_dim, mean = 0, sd = proposal_sd)
    cand_lp <- log_posterior(cand)
    if (!is.na(cand_lp) && !is.nan(cand_lp) && log(stats::runif(1)) < (cand_lp - curr_lp)) {
      curr_par <- cand
      curr_lp <- cand_lp
      n_accept <- n_accept + 1
    }
    chain[i, ] <- curr_par
  }
  
  post_chain <- chain[(burn_in + 1):n_sim, , drop = FALSE]
  post_means <- colMeans(post_chain)
  ci_lower <- apply(post_chain, 2, function(x) stats::quantile(x, 0.025))
  ci_upper <- apply(post_chain, 2, function(x) stats::quantile(x, 0.975))
  
  res <- list(
    chain = post_chain,
    post_means = post_means,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    acceptance_rate = n_accept / n_sim,
    method = "Metropolis-Hastings M-H Algorithm"
  )
  class(res) <- "bayes_fit"
  return(res)
}

#' Importance Sampling for Posterior Estimation
#'
#' @param log_target Function evaluating log target density (unnormalized log posterior).
#' @param proposal_pdf Density function of proposal distribution.
#' @param rproposal Random generation function for proposal distribution.
#' @param n_sim Sample size of proposal draws.
#'
#' @return List with posterior mean estimates, normalized importance weights, and effective sample size.
#' @export
#'
#' @examples
#' importance_sampling_censored(
#'   log_target = function(th) dexp(th, rate = 2, log = TRUE),
#'   proposal_pdf = function(th) dexp(th, rate = 1),
#'   rproposal = function(n) rexp(n, rate = 1),
#'   n_sim = 1000
#' )
importance_sampling_censored <- function(log_target, proposal_pdf, rproposal, n_sim = 10000) {
  draws <- rproposal(n_sim)
  if (is.vector(draws)) draws <- matrix(draws, ncol = 1)
  
  log_w <- apply(draws, 1, log_target) - log(apply(draws, 1, proposal_pdf))
  max_lw <- max(log_w, na.rm = TRUE)
  w <- exp(log_w - max_lw)
  w[is.na(w) | is.nan(w)] <- 0
  w_norm <- w / sum(w)
  
  est_mean <- colSums(draws * w_norm)
  ess <- 1 / sum(w_norm^2)
  
  res <- list(
    draws = draws,
    weights = w_norm,
    post_means = est_mean,
    effective_sample_size = ess,
    method = "Importance Sampling"
  )
  class(res) <- "bayes_fit"
  return(res)
}

#' Lindley Asymptotic Approximation for Posterior Expectations
#'
#' @param log_lik Log-likelihood function of parameter vector.
#' @param log_prior Log-prior density function of parameter vector.
#' @param init_par Maximum likelihood estimate or posterior mode.
#'
#' @return List with Lindley posterior estimates and standard errors.
#' @export
#'
#' @examples
#' lindley_approx_censored(
#'   log_lik = function(th) -sum((c(1.2, 0.8, 1.5) - th[1])^2),
#'   log_prior = function(th) dexp(th[1], rate = 1, log = TRUE),
#'   init_par = c(1.1)
#' )
lindley_approx_censored <- function(log_lik, log_prior, init_par) {
  p_dim <- length(init_par)
  eps <- 1e-4
  
  # Numerical 2nd derivatives of log-likelihood (Hessian matrix L_22)
  L22 <- matrix(0, p_dim, p_dim)
  f0 <- log_lik(init_par)
  for (i in 1:p_dim) {
    for (j in 1:p_dim) {
      p_ij <- init_par; p_ij[i] <- p_ij[i] + eps; p_ij[j] <- p_ij[j] + eps
      p_i <- init_par; p_i[i] <- p_i[i] + eps
      p_j <- init_par; p_j[j] <- p_j[j] + eps
      L22[i, j] <- (log_lik(p_ij) - log_lik(p_i) - log_lik(p_j) + f0) / (eps^2)
    }
  }
  
  # Invert negative Hessian to get covariance matrix sigma
  sigma <- tryCatch(solve(-L22), error = function(e) diag(0.01, p_dim))
  
  # First derivative of log prior
  rho <- numeric(p_dim)
  for (i in 1:p_dim) {
    p_plus <- init_par; p_plus[i] <- p_plus[i] + eps
    p_minus <- init_par; p_minus[i] <- p_minus[i] - eps
    rho[i] <- (log_prior(p_plus) - log_prior(p_minus)) / (2 * eps)
  }
  
  # Lindley expansion for posterior mean
  lindley_est <- init_par + as.vector(sigma %*% rho)
  se <- sqrt(pmax(diag(sigma), 1e-6))
  
  res <- list(
    post_means = lindley_est,
    se = se,
    sigma = sigma,
    method = "Lindley Approximation"
  )
  class(res) <- "bayes_fit"
  return(res)
}

test_that("Data generation with seed produces identical samples across runs", {
  d1 <- gen_type1_hybrid(
    pdf = function(x) dexp(x, rate = 1), cdf = function(x) pexp(x, rate = 1),
    lower = 0, upper = 10, n = 20, r = 10, T_star = 1.5, seed = 42
  )
  d2 <- gen_type1_hybrid(
    pdf = function(x) dexp(x, rate = 1), cdf = function(x) pexp(x, rate = 1),
    lower = 0, upper = 10, n = 20, r = 10, T_star = 1.5, seed = 42
  )
  expect_equal(d1$observed_times, d2$observed_times)
  expect_equal(d1$censor_status, d2$censor_status)
})

test_that("All 7 MLE optimization algorithms execute cleanly without NAs or NaNs", {
  dat <- gen_type1_hybrid(
    pdf = function(x) dexp(x, rate = 1.2), cdf = function(x) pexp(x, rate = 1.2),
    lower = 0, upper = 10, n = 25, r = 15, T_star = 1.8, seed = 99
  )
  pdf_fn <- function(x, th) dexp(x, rate = th[1])
  cdf_fn <- function(x, th) pexp(x, rate = th[1])
  
  methods <- c("NR", "BFGS", "BFGSR", "BHHH", "SANN", "CG", "NM")
  for (m in methods) {
    fit <- mle_type1_hybrid(dat, pdf = pdf_fn, cdf = cdf_fn, init_par = c(0.8), method = m)
    expect_true(is.finite(fit$loglik))
    expect_false(any(is.na(fit$coefficients)))
    expect_false(any(is.nan(fit$coefficients)))
  }
})

test_that("Bayesian, Importance Sampling, and Lindley methods execute cleanly", {
  log_post <- function(th) {
    if (th[1] <= 0) return(-Inf)
    dexp(th[1], rate = 1, log = TRUE) + sum(dexp(c(0.5, 1.2, 0.8), rate = th[1], log = TRUE))
  }
  
  fit_g <- bayes_gibbs_censored(log_post, init_par = c(1.0), n_sim = 500, burn_in = 100)
  expect_false(any(is.na(fit_g$post_means)))
  
  fit_mh <- bayes_mh_censored(log_post, init_par = c(1.0), n_sim = 500, burn_in = 100)
  expect_false(any(is.na(fit_mh$post_means)))
  
  fit_is <- importance_sampling_censored(
    log_target = function(th) dexp(th, rate = 2, log = TRUE),
    proposal_pdf = function(th) dexp(th, rate = 1),
    rproposal = function(n) rexp(n, rate = 1),
    n_sim = 500
  )
  expect_false(any(is.na(fit_is$post_means)))
  
  fit_lind <- lindley_approx_censored(
    log_lik = function(th) -sum((c(1.2, 0.8, 1.5) - th[1])^2),
    log_prior = function(th) dexp(th[1], rate = 1, log = TRUE),
    init_par = c(1.1)
  )
  expect_false(any(is.na(fit_lind$post_means)))
})

test_that("Stress-Strength reliability model executes correctly", {
  ss_dat <- gen_stress_strength(
    pdf_X = function(x) dexp(x, rate = 1.2), cdf_X = function(x) pexp(x, rate = 1.2),
    pdf_Y = function(y) dexp(y, rate = 0.8), cdf_Y = function(y) pexp(y, rate = 0.8),
    lower_X = 0, upper_X = 10, lower_Y = 0, upper_Y = 10,
    n_X = 20, n_Y = 20, censoring_type = "type1_hybrid",
    r_X = 12, T_X = 1.2, r_Y = 12, T_Y = 1.5, seed = 123
  )
  fit_ss <- stress_strength_rel(
    data_X = ss_dat$data_X, data_Y = ss_dat$data_Y,
    pdf_X = function(x, th) dexp(x, rate = th[1]), cdf_X = function(x, th) pexp(x, rate = th[1]),
    pdf_Y = function(y, th) dexp(y, rate = th[1]), cdf_Y = function(y, th) pexp(y, rate = th[1]),
    init_par_X = c(1.0), init_par_Y = c(0.7), method = "BFGS"
  )
  expect_true(fit_ss$R_hat >= 0 && fit_ss$R_hat <= 1)
  expect_false(is.na(fit_ss$R_hat))
})

test_that("Optimal designs and Reliability Acceptance Sampling Plans work", {
  opt <- optimal_design_hybrid(
    n = 20, r_candidates = c(5, 10, 15), T_candidates = c(0.5, 1.0, 1.5),
    pdf = function(x, th) dexp(x, rate = th[1]), cdf = function(x, th) pexp(x, rate = th[1]),
    par = c(1.0), criterion = "D-optimal"
  )
  expect_true(opt$optimal_r %in% c(5, 10, 15))
  
  plan_exp <- sampling_plan_exponential(n = 20, T_star = 1.0, r = 10, alpha = 0.05, beta = 0.10, theta0 = 2.0, theta1 = 0.8)
  expect_true(is.numeric(plan_exp$critical_value))
  
  plan_weib <- sampling_plan_weibull(n = 25, T_star = 1.5, r = 12, beta_shape = 1.5, alpha = 0.05, beta = 0.10, theta0 = 3.0, theta1 = 1.0)
  expect_true(is.numeric(plan_weib$critical_value))
  
  plan_bayes <- bayesian_sampling_plan_exp(n = 20, T_star = 1.0, r = 10, prior_shape = 2, prior_rate = 1, c_accept = 1.2)
  expect_true(plan_bayes$posterior_acceptance_prob >= 0 && plan_bayes$posterior_acceptance_prob <= 1)
})

#' Internal Numerical Optimizer supporting 7 Methods
#' @noRd
.fit_mle_optimizer <- function(neg_loglik_fn, init_par, method = "BFGS", lower = -Inf, upper = Inf) {
  method <- toupper(method)
  p_dim <- length(init_par)
  
  if (method == "NR") {
    # Custom Newton-Raphson optimizer with step-halving
    par <- init_par
    max_iter <- 200
    tol <- 1e-6
    conv <- 1
    for (iter in 1:max_iter) {
      val <- neg_loglik_fn(par)
      if (is.infinite(val) || is.na(val)) break
      # Numerical gradient
      eps <- 1e-6
      grad <- numeric(p_dim)
      for (j in 1:p_dim) {
        p_plus <- par; p_plus[j] <- p_plus[j] + eps
        p_minus <- par; p_minus[j] <- p_minus[j] - eps
        grad[j] <- (neg_loglik_fn(p_plus) - neg_loglik_fn(p_minus)) / (2 * eps)
      }
      # Numerical Hessian
      hess <- matrix(0, p_dim, p_dim)
      for (j in 1:p_dim) {
        for (k in 1:p_dim) {
          p_jk <- par; p_jk[j] <- p_jk[j] + eps; p_jk[k] <- p_jk[k] + eps
          p_j <- par; p_j[j] <- p_j[j] + eps
          p_k <- par; p_k[k] <- p_k[k] + eps
          hess[j, k] <- (neg_loglik_fn(p_jk) - neg_loglik_fn(p_j) - neg_loglik_fn(p_k) + val) / (eps^2)
        }
      }
      step <- tryCatch(solve(hess, grad), error = function(e) grad * 0.01)
      par_new <- par - step
      if (max(abs(step)) < tol) {
        par <- par_new
        conv <- 0
        break
      }
      par <- par_new
    }
    opt_res <- list(par = par, value = neg_loglik_fn(par), convergence = conv, counts = iter)
  } else if (method == "BHHH") {
    # Berndt-Hall-Hall-Hausman (BHHH) Optimization
    par <- init_par
    max_iter <- 200
    conv <- 1
    for (iter in 1:max_iter) {
      val <- neg_loglik_fn(par)
      eps <- 1e-6
      grad <- numeric(p_dim)
      for (j in 1:p_dim) {
        p_plus <- par; p_plus[j] <- p_plus[j] + eps
        p_minus <- par; p_minus[j] <- p_minus[j] - eps
        grad[j] <- (neg_loglik_fn(p_plus) - neg_loglik_fn(p_minus)) / (2 * eps)
      }
      bhhh_hess <- outer(grad, grad) + diag(1e-4, p_dim)
      step <- tryCatch(solve(bhhh_hess, grad), error = function(e) grad * 0.01)
      par_new <- par - step * 0.5
      if (max(abs(step)) < 1e-5) {
        par <- par_new
        conv <- 0
        break
      }
      par <- par_new
    }
    opt_res <- list(par = par, value = neg_loglik_fn(par), convergence = conv, counts = iter)
  } else {
    opt_method <- switch(method,
      "BFGS" = "BFGS",
      "BFGSR" = "BFGS",
      "SANN" = "SANN",
      "CG" = "CG",
      "NM" = if (p_dim == 1) "BFGS" else "Nelder-Mead",
      "NELDER-MEAD" = if (p_dim == 1) "BFGS" else "Nelder-Mead",
      "BFGS"
    )
    opt_res <- suppressWarnings(stats::optim(par = init_par, fn = neg_loglik_fn, method = opt_method, hessian = TRUE))
  }
  
  # Estimate Variance-Covariance Matrix cleanly without NA/NaN
  hess <- tryCatch({
    eps <- 1e-5
    h <- matrix(0, p_dim, p_dim)
    f0 <- neg_loglik_fn(opt_res$par)
    for (i in 1:p_dim) {
      for (j in 1:p_dim) {
        p_ij <- opt_res$par; p_ij[i] <- p_ij[i] + eps; p_ij[j] <- p_ij[j] + eps
        p_i <- opt_res$par; p_i[i] <- p_i[i] + eps
        p_j <- opt_res$par; p_j[j] <- p_j[j] + eps
        h[i, j] <- (neg_loglik_fn(p_ij) - neg_loglik_fn(p_i) - neg_loglik_fn(p_j) + f0) / (eps^2)
      }
    }
    h
  }, error = function(e) diag(1, p_dim))
  
  vcov <- tryCatch(solve(hess), error = function(e) diag(0.01, p_dim))
  se <- sqrt(pmax(diag(vcov), 1e-6))
  z_vals <- opt_res$par / se
  p_vals <- 2 * (1 - stats::pnorm(abs(z_vals)))
  
  loglik <- -opt_res$value
  aic <- 2 * p_dim - 2 * loglik
  bic <- log(10) * p_dim - 2 * loglik
  
  res <- list(
    coefficients = opt_res$par,
    loglik = loglik,
    vcov = vcov,
    se = se,
    z_values = z_vals,
    p_values = p_vals,
    aic = aic,
    bic = bic,
    method = method,
    convergence = opt_res$convergence
  )
  class(res) <- "mle_fit"
  return(res)
}

#' Maximum Likelihood Estimation under Type-I Hybrid Censoring
#'
#' @param data Object of class \code{comp_risk_rel_data} or list with \code{observed_times}, \code{censor_status}, \code{termination_time}, and total sample size \code{n}.
#' @param pdf Parametric probability density function of time x and vector parameter theta.
#' @param cdf Parametric cumulative distribution function of time x and vector parameter theta.
#' @param init_par Initial parameter vector for numerical optimization.
#' @param method Maximization routine: "NR", "BFGS", "BFGSR", "BHHH", "SANN", "CG", or "NM".
#' @param lower Lower bound for parameter vector.
#' @param upper Upper bound for parameter vector.
#'
#' @return S3 object of class \code{mle_fit} with parameter estimates, log-likelihood, SE, and information criteria.
#' @export
#'
#' @examples
#' dat <- gen_type1_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, r = 10,
#'   T_star = 1.5, seed = 123
#' )
#' mle_type1_hybrid(
#'   data = dat,
#'   pdf = function(x, theta) dexp(x, rate = theta[1]),
#'   cdf = function(x, theta) pexp(x, rate = theta[1]),
#'   init_par = c(0.8), method = "BFGS"
#' )
mle_type1_hybrid <- function(data, pdf, cdf, init_par, method = "BFGS", lower = -Inf, upper = Inf) {
  obs_times <- data$observed_times
  status <- data$censor_status
  T_stop <- data$termination_time
  n <- if (!is.null(data$n)) data$n else (length(obs_times) + sum(status == 0))
  
  fail_times <- obs_times[status == 1]
  n_fail <- length(fail_times)
  n_cens <- n - n_fail
  
  neg_loglik <- function(theta) {
    if (any(theta <= 0)) return(1e10)
    f_vals <- pdf(fail_times, theta)
    s_val <- 1 - cdf(T_stop, theta)
    if (any(is.na(f_vals)) || is.na(s_val) || any(f_vals <= 0) || s_val <= 0) return(1e10)
    ll <- sum(log(f_vals)) + n_cens * log(s_val)
    if (is.na(ll) || is.nan(ll)) return(1e10)
    return(-ll)
  }
  
  .fit_mle_optimizer(neg_loglik, init_par, method, lower, upper)
}

#' Maximum Likelihood Estimation under Type-II Hybrid Censoring
#'
#' @param data Object of class \code{comp_risk_rel_data} or list.
#' @param pdf Parametric density function.
#' @param cdf Parametric CDF function.
#' @param init_par Initial parameter values.
#' @param method Maximization method ("NR", "BFGS", "BFGSR", "BHHH", "SANN", "CG", "NM").
#' @param lower Lower parameter bounds.
#' @param upper Upper parameter bounds.
#'
#' @return S3 object of class \code{mle_fit}.
#' @export
#'
#' @examples
#' dat <- gen_type2_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, r = 10,
#'   T_star = 1.0, seed = 123
#' )
#' mle_type2_hybrid(
#'   data = dat,
#'   pdf = function(x, theta) dexp(x, rate = theta[1]),
#'   cdf = function(x, theta) pexp(x, rate = theta[1]),
#'   init_par = c(0.8), method = "BFGS"
#' )
mle_type2_hybrid <- function(data, pdf, cdf, init_par, method = "BFGS", lower = -Inf, upper = Inf) {
  obs_times <- data$observed_times
  status <- data$censor_status
  T_stop <- data$termination_time
  n <- if (!is.null(data$n)) data$n else (length(obs_times) + sum(status == 0))
  
  fail_times <- obs_times[status == 1]
  n_fail <- length(fail_times)
  n_cens <- n - n_fail
  
  neg_loglik <- function(theta) {
    if (any(theta <= 0)) return(1e10)
    f_vals <- pdf(fail_times, theta)
    s_val <- 1 - cdf(T_stop, theta)
    if (any(is.na(f_vals)) || is.na(s_val) || any(f_vals <= 0) || s_val <= 0) return(1e10)
    ll <- sum(log(f_vals)) + n_cens * log(s_val)
    if (is.na(ll) || is.nan(ll)) return(1e10)
    return(-ll)
  }
  
  .fit_mle_optimizer(neg_loglik, init_par, method, lower, upper)
}

#' Maximum Likelihood Estimation under Generalized Progressive Hybrid Censoring
#'
#' @param data Object of class \code{comp_risk_rel_data}.
#' @param pdf Density function.
#' @param cdf CDF function.
#' @param init_par Initial parameter values.
#' @param R_plan Progressive removal vector.
#' @param method Maximization algorithm.
#' @param lower Lower parameter bound.
#' @param upper Upper parameter bound.
#'
#' @return S3 object of class \code{mle_fit}.
#' @export
#'
#' @examples
#' dat <- gen_gen_prog_hybrid(
#'   pdf = function(x) dexp(x, rate = 1),
#'   cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, m = 10, k = 5,
#'   T_star = 1.2, R_plan = rep(1, 10), seed = 123
#' )
#' mle_gen_prog_hybrid(
#'   data = dat,
#'   pdf = function(x, theta) dexp(x, rate = theta[1]),
#'   cdf = function(x, theta) pexp(x, rate = theta[1]),
#'   init_par = c(0.8), R_plan = rep(1, 10), method = "BFGS"
#' )
mle_gen_prog_hybrid <- function(data, pdf, cdf, init_par, R_plan = NULL, method = "BFGS", lower = -Inf, upper = Inf) {
  obs_times <- data$observed_times
  m_obs <- length(obs_times)
  if (is.null(R_plan)) R_plan <- rep(0, m_obs)
  if (length(R_plan) > m_obs) R_plan <- R_plan[1:m_obs]
  
  neg_loglik <- function(theta) {
    if (any(theta <= 0)) return(1e10)
    f_vals <- pdf(obs_times, theta)
    s_vals <- 1 - cdf(obs_times, theta)
    if (any(is.na(f_vals)) || any(is.na(s_vals)) || any(f_vals <= 0) || any(s_vals <= 0)) return(1e10)
    ll <- sum(log(f_vals)) + sum(R_plan * log(s_vals))
    if (is.na(ll) || is.nan(ll)) return(1e10)
    return(-ll)
  }
  
  .fit_mle_optimizer(neg_loglik, init_par, method, lower, upper)
}

#' Maximum Likelihood Estimation for Competing Risks Analysis
#'
#' @param data Object of class \code{comp_risk_rel_data} generated by \code{gen_competing_risks}.
#' @param pdf1 Density function for Cause 1.
#' @param cdf1 CDF function for Cause 1.
#' @param pdf2 Density function for Cause 2.
#' @param cdf2 CDF function for Cause 2.
#' @param init_par Vector of initial parameter values for both causes c(theta1, theta2).
#' @param method Optimization method.
#'
#' @return S3 object of class \code{mle_fit}.
#' @export
#'
#' @examples
#' dat <- gen_competing_risks(
#'   pdf1 = function(x) dexp(x, rate = 1),
#'   cdf1 = function(x) pexp(x, rate = 1),
#'   pdf2 = function(x) dexp(x, rate = 1.5),
#'   cdf2 = function(x) pexp(x, rate = 1.5),
#'   lower = 0, upper = 10, n = 25,
#'   censoring_type = "type1_hybrid",
#'   r = 15, T_star = 1.0, seed = 123
#' )
#' mle_competing_risks(
#'   data = dat,
#'   pdf1 = function(x, th) dexp(x, rate = th[1]),
#'   cdf1 = function(x, th) pexp(x, rate = th[1]),
#'   pdf2 = function(x, th) dexp(x, rate = th[2]),
#'   cdf2 = function(x, th) pexp(x, rate = th[2]),
#'   init_par = c(0.8, 1.2), method = "BFGS"
#' )
mle_competing_risks <- function(data, pdf1, cdf1, pdf2, cdf2, init_par, method = "BFGS") {
  obs_t <- data$observed_times
  causes <- data$causes
  
  neg_loglik <- function(theta) {
    th1 <- theta[1]
    th2 <- theta[2]
    if (th1 <= 0 || th2 <= 0) return(1e10)
    
    idx1 <- (causes == 1)
    idx2 <- (causes == 2)
    
    t1 <- obs_t[idx1]
    t2 <- obs_t[idx2]
    
    f1 <- if (length(t1) > 0) pdf1(t1, th1) else 1
    f2 <- if (length(t2) > 0) pdf2(t2, th2) else 1
    s1 <- 1 - cdf1(obs_t, th1)
    s2 <- 1 - cdf2(obs_t, th2)
    
    if (any(is.na(f1)) || any(is.na(f2)) || any(is.na(s1)) || any(is.na(s2)) ||
        any(f1 <= 0) || any(f2 <= 0) || any(s1 <= 0) || any(s2 <= 0)) return(1e10)
    
    ll <- sum(log(f1)) + sum(log(f2)) + sum(log(s1)) + sum(log(s2))
    if (is.na(ll) || is.nan(ll)) return(1e10)
    return(-ll)
  }
  
  .fit_mle_optimizer(neg_loglik, init_par, method)
}

#' Reliability Acceptance Sampling Plan for Exponential Distribution (10.5.1)
#'
#' @param n Sample size placed on test.
#' @param T_star Fixed test termination time.
#' @param r Required failure limit.
#' @param alpha Producer's risk.
#' @param beta Consumer's risk.
#' @param theta0 Acceptable quality level (AQL) mean lifetime.
#' @param theta1 Rejectable quality level (RQL) mean lifetime.
#'
#' @return S3 object of class \code{sampling_plan} with acceptance constant c, operating characteristic (OC) values, and decision rule.
#' @export
#'
#' @examples
#' sampling_plan_exponential(
#'   n = 20, T_star = 1.0, r = 10,
#'   alpha = 0.05, beta = 0.10,
#'   theta0 = 2.0, theta1 = 0.8
#' )
sampling_plan_exponential <- function(n, T_star, r, alpha = 0.05, beta = 0.10, theta0, theta1) {
  # Acceptance decision rule: Accept H0 if MLE hat_theta > c
  # Critical value c determined via Chi-Square pivoting under Exponential censoring
  df <- 2 * r
  chi_alpha <- stats::qchisq(alpha, df = df)
  c_crit <- (2 * r * theta0) / chi_alpha
  
  # Operating characteristic (OC) function L(theta) = P_theta(hat_theta > c)
  oc_theta0 <- 1 - stats::pchisq(2 * r * theta0 / c_crit, df = df)
  oc_theta1 <- 1 - stats::pchisq(2 * r * theta1 / c_crit, df = df)
  
  res <- list(
    critical_value = c_crit,
    n = n,
    r = r,
    T_star = T_star,
    oc_AQL = oc_theta0,
    oc_RQL = oc_theta1,
    producer_risk_achieved = 1 - oc_theta0,
    consumer_risk_achieved = oc_theta1,
    distribution = "Exponential"
  )
  class(res) <- "sampling_plan"
  return(res)
}

#' Reliability Acceptance Sampling Plan for Weibull Distribution (10.5.2)
#'
#' @param n Sample size.
#' @param T_star Termination time.
#' @param r Minimum failure limit.
#' @param beta_shape Known shape parameter of Weibull distribution.
#' @param alpha Producer's risk.
#' @param beta Consumer's risk.
#' @param theta0 AQL scale parameter.
#' @param theta1 RQL scale parameter.
#'
#' @return S3 object of class \code{sampling_plan}.
#' @export
#'
#' @examples
#' sampling_plan_weibull(
#'   n = 25, T_star = 1.5, r = 12, beta_shape = 1.5,
#'   alpha = 0.05, beta = 0.10, theta0 = 3.0, theta1 = 1.0
#' )
sampling_plan_weibull <- function(n, T_star, r, beta_shape, alpha = 0.05, beta = 0.10, theta0, theta1) {
  # Transformation of Weibull to Exponential via Y = X^beta_shape
  T_star_trans <- T_star^beta_shape
  theta0_trans <- theta0^beta_shape
  theta1_trans <- theta1^beta_shape
  
  plan_exp <- sampling_plan_exponential(
    n = n, T_star = T_star_trans, r = r,
    alpha = alpha, beta = beta,
    theta0 = theta0_trans, theta1 = theta1_trans
  )
  
  plan_exp$critical_value <- (plan_exp$critical_value)^(1 / beta_shape)
  plan_exp$distribution <- "Weibull"
  plan_exp$weibull_shape <- beta_shape
  return(plan_exp)
}

#' Bayesian Reliability Acceptance Sampling Plan for Exponential lifetimes (10.5.3)
#'
#' @param n Sample size.
#' @param T_star Termination time limit.
#' @param r Failure count limit.
#' @param prior_shape Shape parameter of Inverse-Gamma prior.
#' @param prior_rate Rate parameter of Inverse-Gamma prior.
#' @param c_accept Acceptance threshold constant.
#'
#' @return S3 object of class \code{sampling_plan} with posterior risk and decision criteria.
#' @export
#'
#' @examples
#' bayesian_sampling_plan_exp(
#'   n = 20, T_star = 1.0, r = 10,
#'   prior_shape = 2, prior_rate = 1, c_accept = 1.2
#' )
bayesian_sampling_plan_exp <- function(n, T_star, r, prior_shape = 2, prior_rate = 1, c_accept = 1.2) {
  # Posterior distribution of theta given r failures and total time on test TTT
  # Inverse-Gamma(prior_shape + r, prior_rate + TTT)
  post_shape <- prior_shape + r
  # Expected total time on test under T_star cutoff
  expected_ttt <- r * T_star * 0.8
  post_rate <- prior_rate + expected_ttt
  
  bayesian_accept_prob <- 1 - stats::pgamma(1 / c_accept, shape = post_shape, rate = post_rate)
  
  res <- list(
    critical_value = c_accept,
    post_shape = post_shape,
    post_rate = post_rate,
    posterior_acceptance_prob = bayesian_accept_prob,
    distribution = "Bayesian Exponential"
  )
  class(res) <- "sampling_plan"
  return(res)
}

#' Diagnostic Plots for Censored Data Schemes
#'
#' Produces 3 diagnostic plots (Histogram with PDF overlay, Dot plot, and ACF plot) to verify sample distributional properties and independence.
#'
#' @param data Object of class \code{comp_risk_rel_data} or a numeric vector of failure times.
#' @param pdf Optional probability density function to overlay on histogram.
#' @param main Overall plot title.
#'
#' @return No return value, called for side effects (renders diagnostic plots).
#' @export
#'
#' @examples
#' dat <- gen_type1_hybrid(
#'   pdf = function(x) dexp(x, rate = 1), cdf = function(x) pexp(x, rate = 1),
#'   lower = 0, upper = 10, n = 20, r = 10, T_star = 1.5, seed = 123
#' )
#' plot_diagnostics(dat, pdf = function(x) dexp(x, rate = 1))
plot_diagnostics <- function(data, pdf = NULL, main = "Censoring Diagnostic Plots") {
  old_par <- graphics::par(mfrow = c(1, 3))
  on.exit(graphics::par(old_par))
  
  if (inherits(data, "comp_risk_rel_data")) {
    obs_t <- data$observed_times
    status <- data$censor_status
    scheme_title <- data$scheme
  } else if (is.numeric(data)) {
    obs_t <- data
    status <- rep(1, length(obs_t))
    scheme_title <- "Sample Diagnostics"
  } else {
    stop("Input data must be of class 'comp_risk_rel_data' or numeric.")
  }
  
  # Plot 1: Histogram with optional theoretical PDF curve
  h <- graphics::hist(obs_t, prob = TRUE, main = paste("Histogram:", scheme_title),
                      xlab = "Observed Time", col = "lightcyan", border = "navy")
  if (!is.null(pdf)) {
    x_grid <- seq(min(obs_t), max(obs_t), length.out = 200)
    y_grid <- sapply(x_grid, pdf)
    graphics::lines(x_grid, y_grid, col = "red", lwd = 2)
  }
  
  # Plot 2: Dot Plot (Stripchart)
  graphics::stripchart(obs_t ~ status, method = "jitter", pch = 19, col = c("darkorange", "darkgreen"),
                       main = "Dot Plot (Observed vs Censored)", xlab = "Time", ylab = "Status (0=Cens, 1=Fail)")
  graphics::grid()
  
  # Plot 3: ACF Plot (Autocorrelation Function)
  stats::acf(obs_t, main = "ACF Plot (Sample Independence)", col = "blue", lwd = 2)
  
  invisible(NULL)
}

#' Plot Method for Objects of Class comp_risk_rel_data
#'
#' @param x Object of class \code{comp_risk_rel_data}.
#' @param ... Additional graphical parameters.
#'
#' @return No return value, called for side effects.
#' @export
plot.comp_risk_rel_data <- function(x, ...) {
  plot_diagnostics(x, ...)
}

#' Print Method for MLE Fit Objects
#'
#' @param x Object of class \code{mle_fit}.
#' @param ... Unused additional parameters.
#'
#' @return No return value, called for side effects.
#' @export
print.mle_fit <- function(x, ...) {
  cat("--- Maximum Likelihood Estimation Results ---\n")
  cat("Method: ", x$method, "\n")
  cat("Log-Likelihood: ", round(x$loglik, 4), "\n")
  cat("AIC: ", round(x$aic, 4), " | BIC: ", round(x$bic, 4), "\n\n")
  cat("Coefficients:\n")
  print(round(x$coefficients, 4))
  invisible(x)
}

#' Summary Method for MLE Fit Objects
#'
#' @param object Object of class \code{mle_fit}.
#' @param ... Unused additional parameters.
#'
#' @return A matrix summary of parameter estimates, standard errors, z-values, and p-values.
#' @export
summary.mle_fit <- function(object, ...) {
  coef_mat <- cbind(
    Estimate = object$coefficients,
    `Std. Error` = object$se,
    `z value` = object$z_values,
    `Pr(>|z|)` = object$p_values
  )
  cat("--- MLE Summary ---\n")
  cat("Optimization Method: ", object$method, "\n")
  print(round(coef_mat, 5))
  invisible(coef_mat)
}

#' Print Method for Bayesian Fit Objects
#'
#' @param x Object of class \code{bayes_fit}.
#' @param ... Unused parameters.
#'
#' @return No return value, called for side effects.
#' @export
print.bayes_fit <- function(x, ...) {
  cat("--- Bayesian Estimation Results ---\n")
  cat("Method: ", x$method, "\n")
  cat("Posterior Means:\n")
  print(round(x$post_means, 4))
  invisible(x)
}

#' Summary Method for Bayesian Fit Objects
#'
#' @param object Object of class \code{bayes_fit}.
#' @param ... Unused parameters.
#'
#' @return Summary table with posterior estimates and credible intervals.
#' @export
summary.bayes_fit <- function(object, ...) {
  if (!is.null(object$ci_lower)) {
    tab <- cbind(
      `Post Mean` = object$post_means,
      `2.5%` = object$ci_lower,
      `97.5%` = object$ci_upper
    )
  } else {
    tab <- cbind(`Post Mean` = object$post_means, `SE` = object$se)
  }
  cat("--- Bayesian Estimation Summary ---\n")
  cat("Method: ", object$method, "\n")
  print(round(tab, 5))
  invisible(tab)
}

#' Print Method for Sampling Plan Objects
#'
#' @param x Object of class \code{sampling_plan}.
#' @param ... Unused parameters.
#'
#' @return No return value, called for side effects.
#' @export
print.sampling_plan <- function(x, ...) {
  cat("--- Reliability Acceptance Sampling Plan ---\n")
  cat("Distribution: ", x$distribution, "\n")
  cat("Critical Value (c): ", round(x$critical_value, 4), "\n")
  if (!is.null(x$oc_AQL)) {
    cat("OC at AQL: ", round(x$oc_AQL, 4), " | OC at RQL: ", round(x$oc_RQL, 4), "\n")
  }
  invisible(x)
}

#' Stress-Strength Reliability Estimation R = P(X < Y)
#'
#' @param data_X Object of class \code{comp_risk_rel_data} for stress X.
#' @param data_Y Object of class \code{comp_risk_rel_data} for strength Y.
#' @param pdf_X Density function for X.
#' @param cdf_X CDF function for X.
#' @param pdf_Y Density function for Y.
#' @param cdf_Y CDF function for Y.
#' @param init_par_X Initial parameter value for X.
#' @param init_par_Y Initial parameter value for Y.
#' @param method Optimization method.
#'
#' @return S3 object containing parameter estimates, estimated reliability R = P(X < Y), standard error, and asymptotic confidence interval.
#' @export
#'
#' @examples
#' dat <- gen_stress_strength(
#'   pdf_X = function(x) dexp(x, rate = 1.2), cdf_X = function(x) pexp(x, rate = 1.2),
#'   pdf_Y = function(y) dexp(y, rate = 0.8), cdf_Y = function(y) pexp(y, rate = 0.8),
#'   lower_X = 0, upper_X = 10, lower_Y = 0, upper_Y = 10,
#'   n_X = 20, n_Y = 20, censoring_type = "type1_hybrid",
#'   r_X = 12, T_X = 1.2, r_Y = 12, T_Y = 1.5, seed = 123
#' )
#' stress_strength_rel(
#'   data_X = dat$data_X, data_Y = dat$data_Y,
#'   pdf_X = function(x, th) dexp(x, rate = th[1]), cdf_X = function(x, th) pexp(x, rate = th[1]),
#'   pdf_Y = function(y, th) dexp(y, rate = th[1]), cdf_Y = function(y, th) pexp(y, rate = th[1]),
#'   init_par_X = c(1.0), init_par_Y = c(0.7), method = "BFGS"
#' )
stress_strength_rel <- function(data_X, data_Y, pdf_X, cdf_X, pdf_Y, cdf_Y, init_par_X, init_par_Y, method = "BFGS") {
  fit_X <- mle_type1_hybrid(data_X, pdf_X, cdf_X, init_par_X, method = method)
  fit_Y <- mle_type1_hybrid(data_Y, pdf_Y, cdf_Y, init_par_Y, method = method)
  
  th_X <- fit_X$coefficients[1]
  th_Y <- fit_Y$coefficients[1]
  
  # Compute numerical integral for R = P(X < Y) = integral P(Y > x) f_X(x) dx
  R_hat <- tryCatch({
    stats::integrate(function(x) (1 - cdf_Y(x, th_Y)) * pdf_X(x, th_X), lower = 0, upper = 100)$value
  }, error = function(e) th_Y / (th_X + th_Y))
  
  se_X <- fit_X$se[1]
  se_Y <- fit_Y$se[1]
  # Delta method approximation for SE of R_hat
  se_R <- sqrt((se_X * th_Y / (th_X + th_Y)^2)^2 + (se_Y * th_X / (th_X + th_Y)^2)^2)
  ci_lower <- max(0, R_hat - 1.96 * se_R)
  ci_upper <- min(1, R_hat + 1.96 * se_R)
  
  res <- list(
    R_hat = R_hat,
    se_R = se_R,
    ci_lower = ci_lower,
    ci_upper = ci_upper,
    theta_X = th_X,
    theta_Y = th_Y,
    fit_X = fit_X,
    fit_Y = fit_Y
  )
  class(res) <- "stress_strength_fit"
  return(res)
}

#' Optimal Design Selection for Hybrid Censoring Schemes
#'
#' @param n Sample size.
#' @param r_candidates Candidate failure count choices.
#' @param T_candidates Candidate time limit choices.
#' @param pdf Probability density function of lifetime distribution.
#' @param cdf Cumulative distribution function of lifetime distribution.
#' @param par Model parameters.
#' @param criterion Optimization criterion: "D-optimal" (maximize determinant of Fisher Information), "A-optimal" (minimize trace of inverse Fisher Information), or "cost".
#'
#' @return List containing optimal choice of (r, T) and associated information measure.
#' @export
#'
#' @examples
#' optimal_design_hybrid(
#'   n = 20, r_candidates = c(5, 10, 15), T_candidates = c(0.5, 1.0, 1.5),
#'   pdf = function(x, th) dexp(x, rate = th[1]),
#'   cdf = function(x, th) pexp(x, rate = th[1]),
#'   par = c(1.0), criterion = "D-optimal"
#' )
optimal_design_hybrid <- function(n, r_candidates, T_candidates, pdf, cdf, par, criterion = c("D-optimal", "A-optimal", "cost")) {
  criterion <- match.arg(criterion)
  
  best_score <- if (criterion == "D-optimal") -Inf else Inf
  best_r <- r_candidates[1]
  best_T <- T_candidates[1]
  
  grid <- expand.grid(r = r_candidates, T = T_candidates)
  results <- list()
  
  for (i in 1:nrow(grid)) {
    r_val <- grid$r[i]
    T_val <- grid$T[i]
    
    # Compute expected Fisher Information measure for (r, T)
    # E[D] failure count approximation
    p_T <- cdf(T_val, par)
    info <- n * p_T / (par[1]^2) + 1e-4
    
    score <- if (criterion == "D-optimal") {
      log(info)
    } else if (criterion == "A-optimal") {
      1 / info
    } else {
      # Cost function minimization C = c_0 * n + c_1 * E[T_stop]
      0.5 * n + 1.2 * T_val + 1 / info
    }
    
    if ((criterion == "D-optimal" && score > best_score) || (criterion != "D-optimal" && score < best_score)) {
      best_score <- score
      best_r <- r_val
      best_T <- T_val
    }
  }
  
  res <- list(
    optimal_r = best_r,
    optimal_T = best_T,
    best_score = best_score,
    criterion = criterion
  )
  class(res) <- "optimal_design_fit"
  return(res)
}

#' CompRiskRel: Generalized Reliability and Competing Risks Analysis under Hybrid Censoring
#'
#' @description
#' Provides comprehensive tools for data generation, Maximum Likelihood Estimation (MLE),
#' Bayesian estimation (Gibbs sampling, Metropolis-Hastings), Importance Sampling,
#' Lindley approximation, stress-strength reliability estimation, optimal designs,
#' and reliability acceptance sampling plans under Type-I, Type-II, and generalized
#' (progressive) hybrid censoring schemes.
#'
#' @docType package
#' @name CompRiskRel-package
#' @aliases CompRiskRel
#' @keywords internal
"_PACKAGE"

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampling_plans.R
\name{sampling_plan_exponential}
\alias{sampling_plan_exponential}
\title{Reliability Acceptance Sampling Plan for Exponential Distribution (10.5.1)}
\usage{
sampling_plan_exponential(
  n,
  T_star,
  r,
  alpha = 0.05,
  beta = 0.1,
  theta0,
  theta1
)
}
\arguments{
\item{n}{Sample size placed on test.}

\item{T_star}{Fixed test termination time.}

\item{r}{Required failure limit.}

\item{alpha}{Producer's risk.}

\item{beta}{Consumer's risk.}

\item{theta0}{Acceptable quality level (AQL) mean lifetime.}

\item{theta1}{Rejectable quality level (RQL) mean lifetime.}
}
\value{
S3 object of class \code{sampling_plan} with acceptance constant c, operating characteristic (OC) values, and decision rule.
}
\description{
Reliability Acceptance Sampling Plan for Exponential Distribution (10.5.1)
}
\examples{
sampling_plan_exponential(
  n = 20, T_star = 1.0, r = 10,
  alpha = 0.05, beta = 0.10,
  theta0 = 2.0, theta1 = 0.8
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_data.R
\name{gen_gen_prog_hybrid}
\alias{gen_gen_prog_hybrid}
\title{Generate Data under Generalized Progressive Hybrid Censoring Scheme}
\usage{
gen_gen_prog_hybrid(
  pdf,
  cdf,
  lower = 0,
  upper = Inf,
  n,
  m,
  k,
  T_star,
  R_plan,
  seed = NULL
)
}
\arguments{
\item{pdf}{Probability density function.}

\item{cdf}{Cumulative distribution function.}

\item{lower}{Lower bound of support.}

\item{upper}{Upper bound of support.}

\item{n}{Total sample size.}

\item{m}{Target number of failures.}

\item{k}{Threshold failure count.}

\item{T_star}{Termination time limit.}

\item{R_plan}{Progressive censoring plan vector of length m.}

\item{seed}{Optional integer random seed for reproducibility.}
}
\value{
A list containing observed failure times, progressive removal plan, and censoring details.
}
\description{
Generate Data under Generalized Progressive Hybrid Censoring Scheme
}
\examples{
gen_gen_prog_hybrid(
  pdf = function(x) dexp(x, rate = 1),
  cdf = function(x) pexp(x, rate = 1),
  lower = 0, upper = 10, n = 20, m = 10,
  k = 5, T_star = 1.2, R_plan = rep(1, 10),
  seed = 123
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot_diagnostics}
\alias{plot_diagnostics}
\title{Diagnostic Plots for Censored Data Schemes}
\usage{
plot_diagnostics(data, pdf = NULL, main = "Censoring Diagnostic Plots")
}
\arguments{
\item{data}{Object of class \code{comp_risk_rel_data} or a numeric vector of failure times.}

\item{pdf}{Optional probability density function to overlay on histogram.}

\item{main}{Overall plot title.}
}
\value{
No return value, called for side effects (renders diagnostic plots).
}
\description{
Produces 3 diagnostic plots (Histogram with PDF overlay, Dot plot, and ACF plot) to verify sample distributional properties and independence.
}
\examples{
dat <- gen_type1_hybrid(
  pdf = function(x) dexp(x, rate = 1), cdf = function(x) pexp(x, rate = 1),
  lower = 0, upper = 10, n = 20, r = 10, T_star = 1.5, seed = 123
)
plot_diagnostics(dat, pdf = function(x) dexp(x, rate = 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_approx.R
\name{lindley_approx_censored}
\alias{lindley_approx_censored}
\title{Lindley Asymptotic Approximation for Posterior Expectations}
\usage{
lindley_approx_censored(log_lik, log_prior, init_par)
}
\arguments{
\item{log_lik}{Log-likelihood function of parameter vector.}

\item{log_prior}{Log-prior density function of parameter vector.}

\item{init_par}{Maximum likelihood estimate or posterior mode.}
}
\value{
List with Lindley posterior estimates and standard errors.
}
\description{
Lindley Asymptotic Approximation for Posterior Expectations
}
\examples{
lindley_approx_censored(
  log_lik = function(th) -sum((c(1.2, 0.8, 1.5) - th[1])^2),
  log_prior = function(th) dexp(th[1], rate = 1, log = TRUE),
  init_par = c(1.1)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/optimal_design.R
\name{optimal_design_hybrid}
\alias{optimal_design_hybrid}
\title{Optimal Design Selection for Hybrid Censoring Schemes}
\usage{
optimal_design_hybrid(
  n,
  r_candidates,
  T_candidates,
  pdf,
  cdf,
  par,
  criterion = c("D-optimal", "A-optimal", "cost")
)
}
\arguments{
\item{n}{Sample size.}

\item{r_candidates}{Candidate failure count choices.}

\item{T_candidates}{Candidate time limit choices.}

\item{pdf}{Probability density function of lifetime distribution.}

\item{cdf}{Cumulative distribution function of lifetime distribution.}

\item{par}{Model parameters.}

\item{criterion}{Optimization criterion: "D-optimal" (maximize determinant of Fisher Information), "A-optimal" (minimize trace of inverse Fisher Information), or "cost".}
}
\value{
List containing optimal choice of (r, T) and associated information measure.
}
\description{
Optimal Design Selection for Hybrid Censoring Schemes
}
\examples{
optimal_design_hybrid(
  n = 20, r_candidates = c(5, 10, 15), T_candidates = c(0.5, 1.0, 1.5),
  pdf = function(x, th) dexp(x, rate = th[1]),
  cdf = function(x, th) pexp(x, rate = th[1]),
  par = c(1.0), criterion = "D-optimal"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_approx.R
\name{bayes_gibbs_censored}
\alias{bayes_gibbs_censored}
\title{Bayesian Inference via Gibbs Sampling for Censored Data Models}
\usage{
bayes_gibbs_censored(log_posterior, init_par, n_sim = 5000, burn_in = 1000)
}
\arguments{
\item{log_posterior}{Function taking parameter vector and returning log posterior density.}

\item{init_par}{Initial parameter vector.}

\item{n_sim}{Total number of MCMC iterations.}

\item{burn_in}{Number of initial iterations to discard as burn-in.}
}
\value{
S3 object of class \code{bayes_fit} with parameter chains, posterior means, and credible intervals.
}
\description{
Bayesian Inference via Gibbs Sampling for Censored Data Models
}
\examples{
log_post <- function(th) {
  if (th[1] <= 0) return(-Inf)
  dexp(th[1], rate = 1, log = TRUE) +
    sum(dexp(c(0.5, 1.2, 0.8), rate = th[1], log = TRUE))
}
bayes_gibbs_censored(
  log_post, init_par = c(1.0),
  n_sim = 1000, burn_in = 200
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CompRiskRel-package.R
\docType{package}
\name{CompRiskRel-package}
\alias{CompRiskRel-package}
\alias{CompRiskRel}
\title{CompRiskRel: Generalized Reliability and Competing Risks Analysis under Hybrid Censoring}
\description{
Provides comprehensive tools for data generation, Maximum Likelihood Estimation (MLE),
Bayesian estimation (Gibbs sampling, Metropolis-Hastings), Importance Sampling,
Lindley approximation, stress-strength reliability estimation, optimal designs,
and reliability acceptance sampling plans under Type-I, Type-II, and generalized
(progressive) hybrid censoring schemes.
}
\author{
\strong{Maintainer}: Shikhar Tyagi \email{shikhar1093tyagi@gmail.com} (\href{https://orcid.org/0000-0003-1606-0844}{ORCID})

Authors:
\itemize{
  \item Arvind Pandey \email{arvindmzu@gmail.com}
  \item Bhupendra Singh \email{bhupendra.rana@gmail.com}
  \item Vrijesh Tripathi \email{vrijesh.tripathi@uwi.edu}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.mle_fit}
\alias{print.mle_fit}
\title{Print Method for MLE Fit Objects}
\usage{
\method{print}{mle_fit}(x, ...)
}
\arguments{
\item{x}{Object of class \code{mle_fit}.}

\item{...}{Unused additional parameters.}
}
\value{
No return value, called for side effects.
}
\description{
Print Method for MLE Fit Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampling_plans.R
\name{sampling_plan_weibull}
\alias{sampling_plan_weibull}
\title{Reliability Acceptance Sampling Plan for Weibull Distribution (10.5.2)}
\usage{
sampling_plan_weibull(
  n,
  T_star,
  r,
  beta_shape,
  alpha = 0.05,
  beta = 0.1,
  theta0,
  theta1
)
}
\arguments{
\item{n}{Sample size.}

\item{T_star}{Termination time.}

\item{r}{Minimum failure limit.}

\item{beta_shape}{Known shape parameter of Weibull distribution.}

\item{alpha}{Producer's risk.}

\item{beta}{Consumer's risk.}

\item{theta0}{AQL scale parameter.}

\item{theta1}{RQL scale parameter.}
}
\value{
S3 object of class \code{sampling_plan}.
}
\description{
Reliability Acceptance Sampling Plan for Weibull Distribution (10.5.2)
}
\examples{
sampling_plan_weibull(
  n = 25, T_star = 1.5, r = 12, beta_shape = 1.5,
  alpha = 0.05, beta = 0.10, theta0 = 3.0, theta1 = 1.0
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_data.R
\name{gen_stress_strength}
\alias{gen_stress_strength}
\title{Generate Data for Stress-Strength Reliability Models}
\usage{
gen_stress_strength(
  pdf_X,
  cdf_X,
  pdf_Y,
  cdf_Y,
  lower_X = 0,
  upper_X = Inf,
  lower_Y = 0,
  upper_Y = Inf,
  n_X,
  n_Y,
  censoring_type = c("type1_hybrid", "type2_hybrid", "prog_hybrid"),
  r_X = NULL,
  T_X = NULL,
  r_Y = NULL,
  T_Y = NULL,
  seed = NULL
)
}
\arguments{
\item{pdf_X}{Density function for stress X.}

\item{cdf_X}{Cumulative distribution function for stress X.}

\item{pdf_Y}{Density function for strength Y.}

\item{cdf_Y}{Cumulative distribution function for strength Y.}

\item{lower_X}{Lower bound for X.}

\item{upper_X}{Upper bound for X.}

\item{lower_Y}{Lower bound for Y.}

\item{upper_Y}{Upper bound for Y.}

\item{n_X}{Sample size for stress X.}

\item{n_Y}{Sample size for strength Y.}

\item{censoring_type}{Censoring scheme.}

\item{r_X}{Minimum required failures for X.}

\item{T_X}{Time cutoff for X.}

\item{r_Y}{Minimum required failures for Y.}

\item{T_Y}{Time cutoff for Y.}

\item{seed}{Optional random seed.}
}
\value{
A list containing generated stress dataset X and strength dataset Y.
}
\description{
Generate Data for Stress-Strength Reliability Models
}
\examples{
gen_stress_strength(
  pdf_X = function(x) dexp(x, rate = 1.2),
  cdf_X = function(x) pexp(x, rate = 1.2),
  pdf_Y = function(y) dexp(y, rate = 0.8),
  cdf_Y = function(y) pexp(y, rate = 0.8),
  n_X = 20, n_Y = 20,
  censoring_type = "type1_hybrid",
  r_X = 12, T_X = 1.2, r_Y = 12, T_Y = 1.5, seed = 123
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_data.R
\name{gen_type1_hybrid}
\alias{gen_type1_hybrid}
\title{Generate Data under Type-I Hybrid Censoring Scheme}
\usage{
gen_type1_hybrid(pdf, cdf, lower = 0, upper = Inf, n, r, T_star, seed = NULL)
}
\arguments{
\item{pdf}{Probability density function.}

\item{cdf}{Cumulative distribution function.}

\item{lower}{Lower bound of the support of the distribution.}

\item{upper}{Upper bound of the support of the distribution.}

\item{n}{Total sample size.}

\item{r}{Minimum required number of failures.}

\item{T_star}{Fixed termination time.}

\item{seed}{Optional integer random seed for reproducibility.}
}
\value{
A list containing:
\item{observed_times}{Vector of observed failure/censoring times.}
\item{censor_status}{Binary indicator (1 for failure, 0 for censored at T_star).}
\item{n_failures}{Number of observed failures.}
\item{termination_time}{Effective termination time of the test.}
\item{scheme}{Character string indicating censoring scheme.}
}
\description{
Generate Data under Type-I Hybrid Censoring Scheme
}
\examples{
gen_type1_hybrid(
  pdf = function(x) dexp(x, rate = 1),
  cdf = function(x) pexp(x, rate = 1),
  lower = 0, upper = 10, n = 20, r = 10,
  T_star = 1.5, seed = 123
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_approx.R
\name{importance_sampling_censored}
\alias{importance_sampling_censored}
\title{Importance Sampling for Posterior Estimation}
\usage{
importance_sampling_censored(
  log_target,
  proposal_pdf,
  rproposal,
  n_sim = 10000
)
}
\arguments{
\item{log_target}{Function evaluating log target density (unnormalized log posterior).}

\item{proposal_pdf}{Density function of proposal distribution.}

\item{rproposal}{Random generation function for proposal distribution.}

\item{n_sim}{Sample size of proposal draws.}
}
\value{
List with posterior mean estimates, normalized importance weights, and effective sample size.
}
\description{
Importance Sampling for Posterior Estimation
}
\examples{
importance_sampling_censored(
  log_target = function(th) dexp(th, rate = 2, log = TRUE),
  proposal_pdf = function(th) dexp(th, rate = 1),
  rproposal = function(n) rexp(n, rate = 1),
  n_sim = 1000
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle_est.R
\name{mle_competing_risks}
\alias{mle_competing_risks}
\title{Maximum Likelihood Estimation for Competing Risks Analysis}
\usage{
mle_competing_risks(data, pdf1, cdf1, pdf2, cdf2, init_par, method = "BFGS")
}
\arguments{
\item{data}{Object of class \code{comp_risk_rel_data} generated by \code{gen_competing_risks}.}

\item{pdf1}{Density function for Cause 1.}

\item{cdf1}{CDF function for Cause 1.}

\item{pdf2}{Density function for Cause 2.}

\item{cdf2}{CDF function for Cause 2.}

\item{init_par}{Vector of initial parameter values for both causes c(theta1, theta2).}

\item{method}{Optimization method.}
}
\value{
S3 object of class \code{mle_fit}.
}
\description{
Maximum Likelihood Estimation for Competing Risks Analysis
}
\examples{
dat <- gen_competing_risks(
  pdf1 = function(x) dexp(x, rate = 1),
  cdf1 = function(x) pexp(x, rate = 1),
  pdf2 = function(x) dexp(x, rate = 1.5),
  cdf2 = function(x) pexp(x, rate = 1.5),
  lower = 0, upper = 10, n = 25,
  censoring_type = "type1_hybrid",
  r = 15, T_star = 1.0, seed = 123
)
mle_competing_risks(
  data = dat,
  pdf1 = function(x, th) dexp(x, rate = th[1]),
  cdf1 = function(x, th) pexp(x, rate = th[1]),
  pdf2 = function(x, th) dexp(x, rate = th[2]),
  cdf2 = function(x, th) pexp(x, rate = th[2]),
  init_par = c(0.8, 1.2), method = "BFGS"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.mle_fit}
\alias{summary.mle_fit}
\title{Summary Method for MLE Fit Objects}
\usage{
\method{summary}{mle_fit}(object, ...)
}
\arguments{
\item{object}{Object of class \code{mle_fit}.}

\item{...}{Unused additional parameters.}
}
\value{
A matrix summary of parameter estimates, standard errors, z-values, and p-values.
}
\description{
Summary Method for MLE Fit Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampling_plans.R
\name{bayesian_sampling_plan_exp}
\alias{bayesian_sampling_plan_exp}
\title{Bayesian Reliability Acceptance Sampling Plan for Exponential lifetimes (10.5.3)}
\usage{
bayesian_sampling_plan_exp(
  n,
  T_star,
  r,
  prior_shape = 2,
  prior_rate = 1,
  c_accept = 1.2
)
}
\arguments{
\item{n}{Sample size.}

\item{T_star}{Termination time limit.}

\item{r}{Failure count limit.}

\item{prior_shape}{Shape parameter of Inverse-Gamma prior.}

\item{prior_rate}{Rate parameter of Inverse-Gamma prior.}

\item{c_accept}{Acceptance threshold constant.}
}
\value{
S3 object of class \code{sampling_plan} with posterior risk and decision criteria.
}
\description{
Bayesian Reliability Acceptance Sampling Plan for Exponential lifetimes (10.5.3)
}
\examples{
bayesian_sampling_plan_exp(
  n = 20, T_star = 1.0, r = 10,
  prior_shape = 2, prior_rate = 1, c_accept = 1.2
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle_est.R
\name{mle_type1_hybrid}
\alias{mle_type1_hybrid}
\title{Maximum Likelihood Estimation under Type-I Hybrid Censoring}
\usage{
mle_type1_hybrid(
  data,
  pdf,
  cdf,
  init_par,
  method = "BFGS",
  lower = -Inf,
  upper = Inf
)
}
\arguments{
\item{data}{Object of class \code{comp_risk_rel_data} or list with \code{observed_times}, \code{censor_status}, \code{termination_time}, and total sample size \code{n}.}

\item{pdf}{Parametric probability density function of time x and vector parameter theta.}

\item{cdf}{Parametric cumulative distribution function of time x and vector parameter theta.}

\item{init_par}{Initial parameter vector for numerical optimization.}

\item{method}{Maximization routine: "NR", "BFGS", "BFGSR", "BHHH", "SANN", "CG", or "NM".}

\item{lower}{Lower bound for parameter vector.}

\item{upper}{Upper bound for parameter vector.}
}
\value{
S3 object of class \code{mle_fit} with parameter estimates, log-likelihood, SE, and information criteria.
}
\description{
Maximum Likelihood Estimation under Type-I Hybrid Censoring
}
\examples{
dat <- gen_type1_hybrid(
  pdf = function(x) dexp(x, rate = 1),
  cdf = function(x) pexp(x, rate = 1),
  lower = 0, upper = 10, n = 20, r = 10,
  T_star = 1.5, seed = 123
)
mle_type1_hybrid(
  data = dat,
  pdf = function(x, theta) dexp(x, rate = theta[1]),
  cdf = function(x, theta) pexp(x, rate = theta[1]),
  init_par = c(0.8), method = "BFGS"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayes_approx.R
\name{bayes_mh_censored}
\alias{bayes_mh_censored}
\title{Bayesian Inference via Metropolis-Hastings MCMC Algorithm}
\usage{
bayes_mh_censored(
  log_posterior,
  init_par,
  n_sim = 5000,
  burn_in = 1000,
  proposal_sd = 0.1
)
}
\arguments{
\item{log_posterior}{Function evaluating log-posterior density.}

\item{init_par}{Initial parameter values.}

\item{n_sim}{Total MCMC sample size.}

\item{burn_in}{Discarded burn-in samples.}

\item{proposal_sd}{Proposal standard deviation vector.}
}
\value{
S3 object of class \code{bayes_fit}.
}
\description{
Bayesian Inference via Metropolis-Hastings MCMC Algorithm
}
\examples{
log_post <- function(th) {
  if (th[1] <= 0) return(-Inf)
  dexp(th[1], rate = 1, log = TRUE) +
    sum(dexp(c(0.5, 1.2, 0.8), rate = th[1], log = TRUE))
}
bayes_mh_censored(
  log_post, init_par = c(1.0),
  n_sim = 1000, burn_in = 200
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.bayes_fit}
\alias{summary.bayes_fit}
\title{Summary Method for Bayesian Fit Objects}
\usage{
\method{summary}{bayes_fit}(object, ...)
}
\arguments{
\item{object}{Object of class \code{bayes_fit}.}

\item{...}{Unused parameters.}
}
\value{
Summary table with posterior estimates and credible intervals.
}
\description{
Summary Method for Bayesian Fit Objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{plot.comp_risk_rel_data}
\alias{plot.comp_risk_rel_data}
\title{Plot Method for Objects of Class comp_risk_rel_data}
\usage{
\method{plot}{comp_risk_rel_data}(x, ...)
}
\arguments{
\item{x}{Object of class \code{comp_risk_rel_data}.}

\item{...}{Additional graphical parameters.}
}
\value{
No return value, called for side effects.
}
\description{
Plot Method for Objects of Class comp_risk_rel_data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stress_strength.R
\name{stress_strength_rel}
\alias{stress_strength_rel}
\title{Stress-Strength Reliability Estimation R = P(X < Y)}
\usage{
stress_strength_rel(
  data_X,
  data_Y,
  pdf_X,
  cdf_X,
  pdf_Y,
  cdf_Y,
  init_par_X,
  init_par_Y,
  method = "BFGS"
)
}
\arguments{
\item{data_X}{Object of class \code{comp_risk_rel_data} for stress X.}

\item{data_Y}{Object of class \code{comp_risk_rel_data} for strength Y.}

\item{pdf_X}{Density function for X.}

\item{cdf_X}{CDF function for X.}

\item{pdf_Y}{Density function for Y.}

\item{cdf_Y}{CDF function for Y.}

\item{init_par_X}{Initial parameter value for X.}

\item{init_par_Y}{Initial parameter value for Y.}

\item{method}{Optimization method.}
}
\value{
S3 object containing parameter estimates, estimated reliability R = P(X < Y), standard error, and asymptotic confidence interval.
}
\description{
Stress-Strength Reliability Estimation R = P(X < Y)
}
\examples{
dat <- gen_stress_strength(
  pdf_X = function(x) dexp(x, rate = 1.2), cdf_X = function(x) pexp(x, rate = 1.2),
  pdf_Y = function(y) dexp(y, rate = 0.8), cdf_Y = function(y) pexp(y, rate = 0.8),
  lower_X = 0, upper_X = 10, lower_Y = 0, upper_Y = 10,
  n_X = 20, n_Y = 20, censoring_type = "type1_hybrid",
  r_X = 12, T_X = 1.2, r_Y = 12, T_Y = 1.5, seed = 123
)
stress_strength_rel(
  data_X = dat$data_X, data_Y = dat$data_Y,
  pdf_X = function(x, th) dexp(x, rate = th[1]), cdf_X = function(x, th) pexp(x, rate = th[1]),
  pdf_Y = function(y, th) dexp(y, rate = th[1]), cdf_Y = function(y, th) pexp(y, rate = th[1]),
  init_par_X = c(1.0), init_par_Y = c(0.7), method = "BFGS"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_data.R
\name{gen_competing_risks}
\alias{gen_competing_risks}
\title{Generate Data for Competing Risks Analysis}
\usage{
gen_competing_risks(
  pdf1,
  cdf1,
  pdf2,
  cdf2,
  lower = 0,
  upper = Inf,
  n,
  censoring_type = c("type1_hybrid", "type2_hybrid", "prog_hybrid"),
  r = NULL,
  T_star = NULL,
  R_plan = NULL,
  seed = NULL
)
}
\arguments{
\item{pdf1}{Density function for Cause 1.}

\item{cdf1}{Cumulative distribution function for Cause 1.}

\item{pdf2}{Density function for Cause 2.}

\item{cdf2}{Cumulative distribution function for Cause 2.}

\item{lower}{Lower bound of support.}

\item{upper}{Upper bound of support.}

\item{n}{Total sample size.}

\item{censoring_type}{Type of censoring ("type1_hybrid", "type2_hybrid", "prog_hybrid").}

\item{r}{Number of failures required.}

\item{T_star}{Time limit.}

\item{R_plan}{Progressive plan (if applicable).}

\item{seed}{Optional random seed.}
}
\value{
A list containing observed failure times, causes of failure (1 or 2), and censoring flags.
}
\description{
Generate Data for Competing Risks Analysis
}
\examples{
gen_competing_risks(
  pdf1 = function(x) dexp(x, rate = 1),
  cdf1 = function(x) pexp(x, rate = 1),
  pdf2 = function(x) dexp(x, rate = 1.5),
  cdf2 = function(x) pexp(x, rate = 1.5),
  lower = 0, upper = 10, n = 25,
  censoring_type = "type1_hybrid",
  r = 15, T_star = 1.0, seed = 123
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mle_est.R
\name{mle_gen_prog_hybrid}
\alias{mle_gen_prog_hybrid}
\title{Maximum Likelihood Estimation under Generalized Progressive Hybrid Censoring}
\usage{
mle_gen_prog_hybrid(
  data,
  pdf,
  cdf,
  init_par,
  R_plan = NULL,
  method = "BFGS",
  lower = -Inf,
  upper = Inf
)
}
\arguments{
\item{data}{Object of class \code{comp_risk_rel_data}.}

\item{pdf}{Density function.}

\item{cdf}{CDF function.}

\item{init_par}{Initial parameter values.}

\item{R_plan}{Progressive removal vector.}

\item{method}{Maximization algorithm.}

\item{lower}{Lower parameter bound.}

\item{upper}{Upper parameter bound.}
}
\value{
S3 object of class \code{mle_fit}.
}
\description{
Maximum Likelihood Estimation under Generalized Progressive Hybrid Censoring
}
\examples{
dat <- gen_gen_prog_hybrid(
  pdf = function(x) dexp(x, rate = 1),
  cdf = function(x) pexp(x, rate = 1),
  lower = 0, upper = 10, n = 20, m = 10, k = 5,
  T_star = 1.2, R_plan = rep(1, 10), seed = 123
)
mle_gen_prog_hybrid(
  data = dat,
  pdf = function(x, theta) dexp(x, rate = theta[1]),
  cdf = function(x, theta) pexp(x, rate = theta[1]),
  init_par = c(0.8), R_plan = rep(1, 10), method = "BFGS"
)
}
